<div class="modal fade higherWider" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
    <div class="modal-dialog error-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal">
                    <span aria-hidden="true">&times;</span>
                    <span class="sr-only">Close</span>
                </button>
                <h4 class="modal-title"><%- _(title).t() %></h4>
            </div>
            <div class="modal-body">
                <div class="msg msg-error">
                    <div class="msg-body">
                        <i class="icon icon-alert-circle"></i>
                        <div class="msg-text"><%- _(msg).t() %></div>
                    </div>
                </div>
            </div>
            <div class="modal-footer">
                <button type="button" class="btn btn-primary" data-dismiss="modal">
                    <%- _("OK").t() %>
                </button>
            </div>
        </div>
    </div>
</div>
