<div class ="tool-bar">
  <div class ="tool-title">
    <span class="icon-tasks"></span><%- _(title).t() %>
  </div>
  <% if (enableButton) { %>
    <div class = "tool-button">
      <div class="dropdown add-button">
        <button  id="<%= buttonId%>" class="btn btn-primary" type="button">
           <%- _(buttonValue).t() %>
          <% if (!singleInput) { %>
            <span class="caret"></span>
          <% } %>
        </button>
      </div>
    </div>
  <% } %>

    <div class= "tool-description">
        <%- _(description).t() %>
    </div>
</div>
<% if(enableHr){ %>
<hr>
<% } %>
