<form class="form form-horizontal" role="form" onsubmit="return false;" autocomplete="off">
    <div class="modal-body">
    </div>
    <div class="panel-footer">
        <div class="form-horizontal form-small">
            <div class="form-group control-group">
                <div class="col-sm-10 controls control-placeholder">
                    <input type="submit" id="<%- buttonId %>" class="btn btn-primary submit-btn" value=<%- _("Save").t() %> >
                </div>
            </div>
        </div>
    </div>
</form>
