<div class="add-button">
    <div class="btn-group btn-group-sm">
        <button type="button" class="btn add-button btn-primary" id="<%- buttonId %>">
            <%- _(buttonValue).t() %>
        </button>
    </div>
</div>
