/*Horizon line*/
.main-section-body hr {
    margin: 0px;
    border-bottom: none;
    border-top: 1px solid #ccc;
}

/*Input type filter*/
.type-filter {
    float: left;
    margin-top: 10px;
    margin-left: 30px;
}

.list-dotted dd {
    min-height: 20px;
}

/*Page count indicator*/
.table-caption-inner > .shared-controls-syntheticselectcontrol {
    float: left;
    margin-top: 10px;
}

/*Input filter scroll style fix for Safari*/
.dropdown-menu ul {
    overflow-y: scroll !important;
}