<div class ="tool-bar">
    <div class ="tool-title"><%- _(title).t() %></div>
    <div class = "tool-button">
        <div class="dropdown add-button">
            <button id="addInputBtn" class="btn btn-primary" type="button">
                <%- _(buttonText).t() %>
                <% if (!singleInput) { %>
                    <span class="caret"></span>
                <% } %>
            </button>
        </div>
    </div>
    <div class= "tool-description">
        <%- _(description).t() %>
    </div>
</div>
<hr>
